package com.engine.organization.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.api.browser.bean.BrowserBean;
import com.api.browser.bean.SearchConditionGroup;
import com.api.browser.bean.SearchConditionItem;
import com.api.browser.bean.SearchConditionOption;
import com.cloudstore.eccom.pc.table.WeaTableColumn;
import com.cloudstore.eccom.result.WeaResultMsg;
import com.engine.common.util.ServiceUtil;
import com.engine.core.impl.Service;
import com.engine.organization.component.OrganizationWeaTable;
import com.engine.organization.entity.company.bo.CompBO;
import com.engine.organization.entity.company.dto.CompListDTO;
import com.engine.organization.entity.company.param.CompSearchParam;
import com.engine.organization.entity.company.po.CompPO;
import com.engine.organization.mapper.comp.CompMapper;
import com.engine.organization.mapper.extend.ExtendGroupMapper;
import com.engine.organization.service.CompService;
import com.engine.organization.service.ExtService;
import com.engine.organization.util.MenuBtn;
import com.engine.organization.util.OrganizationAssert;
import com.engine.organization.util.OrganizationFormItemUtil;
import com.engine.organization.util.db.MapperProxyFactory;
import com.engine.organization.util.page.Column;
import com.engine.organization.util.page.PageInfo;
import com.engine.organization.util.page.PageUtil;
import weaver.crm.Maint.SectorInfoComInfo;
import weaver.general.StringUtil;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.hrm.resource.ResourceComInfo;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @description: TODO
 * @author:dxfeng
 * @createTime: 2022/05/16
 * @version: 1.0
 */
public class CompServiceImpl extends Service implements CompService {
    /**
     * 分组类型
     * 1：分部
     * 2：部门
     * 3：岗位
     */
    private static final String EXTEND_TYPE = "1";
    /**
     * 主表拓展表
     */
    private static final String JCL_ORG_COMPEXT = "JCL_ORG_COMPEXT";
    /**
     * 明细表拓展表
     */
    private static final String JCL_ORG_COMPEXT_DT1 = "JCL_ORG_COMPEXT_DT1";

    private CompMapper getCompMapper() {
        return MapperProxyFactory.getProxy(CompMapper.class);
    }

    private ExtendGroupMapper getExtendGroupMapper() {
        return MapperProxyFactory.getProxy(ExtendGroupMapper.class);
    }

    private ExtService getExtService(User user) {
        return ServiceUtil.getService(ExtServiceImpl.class, user);
    }


    @Override
    public Map<String, Object> listPage(CompSearchParam params) {
        Map<String, Object> datas = new HashMap<>();
        CompPO compPO = CompBO.convertParamToPO(params, (long) user.getUID());
        boolean filter = isFilter(compPO);
        PageInfo<CompListDTO> pageInfos;
        List<CompPO> alltList = getCompMapper().list();
        // 通过子级遍历父级元素
        if (filter) {
            // 根据条件获取元素
            List<CompPO> filterCompPOs = getCompMapper().listByFilter(compPO);
            // 添加父级元素
            List<CompListDTO> compListDTOS = CompBO.buildCompDTOList(alltList, filterCompPOs);
            List<CompListDTO> subList = PageUtil.subList(params.getCurrent(), params.getPageSize(), compListDTOS);
            pageInfos = new PageInfo<>(subList, CompListDTO.class);
            pageInfos.setTotal(compListDTOS.size());
        } else {
            // 组合list
            List<CompListDTO> compListDTOS = CompBO.buildCompDTOList(alltList);
            List<CompListDTO> subList = PageUtil.subList(params.getCurrent(), params.getPageSize(), compListDTOS);
            pageInfos = new PageInfo<>(subList, CompListDTO.class);
            pageInfos.setTotal(compListDTOS.size());
        }

        pageInfos.setPageNum(params.getCurrent());
        pageInfos.setPageSize(params.getPageSize());

        OrganizationWeaTable<CompListDTO> table = new OrganizationWeaTable<>(user, CompListDTO.class);
        List<Column> columns = pageInfos.getColumns();
        List<WeaTableColumn> weaTableColumn = columns.stream().map(v -> new WeaTableColumn("100", v.getTitle(), v.getKey())).collect(Collectors.toList());

        table.setColumns(weaTableColumn);

        WeaResultMsg result = new WeaResultMsg(false);
        result.putAll(table.makeDataResult());
        result.success();

        datas.put("pageInfo", pageInfos);
        datas.put("dataKey", result.getResultMap());
        return datas;
    }

    @Override
    public int saveBaseComp(CompSearchParam params) {
        List<CompPO> list = getCompMapper().listByNo(Util.null2String(params.getCompNo()));
        OrganizationAssert.isEmpty(list, "编号不允许重复");
        CompPO compPO = CompBO.convertParamToPO(params, (long) user.getUID());
        return getCompMapper().insertIgnoreNull(compPO);
    }

    @Override
    public int updateForbiddenTagById(CompSearchParam params) {
        CompPO compPO = CompPO.builder().id(params.getId()).forbiddenTag(params.getForbiddenTag() ? 0 : 1).build();
        return getCompMapper().updateForbiddenTagById(compPO);
    }

    @Override
    public int updateComp(Map<String, Object> params) {
        CompSearchParam searchParam = JSONObject.parseObject(JSONObject.toJSONString(params), CompSearchParam.class);
        String groupId = (String) params.get("viewCondition");
        CompPO compPO = CompBO.convertParamToPO(searchParam, (long) user.getUID());
        int updateCount = 0;
        // 更新主表数据
        updateCount += getCompMapper().updateBaseComp(compPO);
        // 更新主表拓展表
        updateCount += getExtService(user).updateExtForm(user, EXTEND_TYPE, JCL_ORG_COMPEXT, params, groupId, compPO.getId());
        //更新明细表
        getExtService(user).updateExtDT(user, EXTEND_TYPE, JCL_ORG_COMPEXT_DT1, params, compPO.getId());

        return updateCount;
    }

    @Override
    public int deleteByIds(Collection<Long> ids) {
        OrganizationAssert.notEmpty(ids, "请选择要删除的数据");
        return getCompMapper().deleteByIds(ids);
    }

    @Override
    public Map<String, Object> getSearchCondition(Map<String, Object> params) {
        Map<String, Object> apiDatas = new HashMap<>();
        List<SearchConditionGroup> addGroups = new ArrayList<>();
        List<SearchConditionItem> conditionItems = new ArrayList<>();
        // 编号
        SearchConditionItem compNoItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 2, 50, "编号", "compNo");
        // 名称
        SearchConditionItem compNameItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 2, 50, "名称", "compName");
        // 简称
        SearchConditionItem compNameShortItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 2, 50, "简称", "compNameShort");
        // TODO
        // 上级公司
        SearchConditionItem compBrowserItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 2, false, "上级公司", "161", "parentCompany", "compBrowser");
        // 组织机构代码
        SearchConditionItem orgCodeItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 2, 50, "组织机构代码", "orgCode");
        // 行业
        SearchConditionItem industryItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 2, false, "行业", "63", "industry", "");
        // 负责人
        SearchConditionItem compPrincipalItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 2, false, "负责人", "1", "compPrincipal", "");
        // 禁用标记
        List<SearchConditionOption> selectOptions = new ArrayList<>();
        SearchConditionOption enableOption = new SearchConditionOption("true", "启用");
        SearchConditionOption disableOption = new SearchConditionOption("false", "禁用");
        selectOptions.add(enableOption);
        selectOptions.add(disableOption);
        SearchConditionItem forbiddenTagItem = OrganizationFormItemUtil.selectItem(user, selectOptions, 2, 16, 6, false, "禁用标记", "forbiddenTag");

        conditionItems.add(compNoItem);
        conditionItems.add(compNameItem);
        conditionItems.add(compNameShortItem);
        conditionItems.add(compBrowserItem);
        conditionItems.add(orgCodeItem);
        conditionItems.add(industryItem);
        conditionItems.add(compPrincipalItem);
        conditionItems.add(forbiddenTagItem);

        addGroups.add(new SearchConditionGroup("高级搜索条件", true, conditionItems));
        apiDatas.put("conditions", addGroups);
        return apiDatas;
    }

    @Override
    public Map<String, Object> getHasRight() {
        Map<String, Object> btnDatas = new HashMap<>();
        ArrayList<MenuBtn> topMenuList = new ArrayList<>();
        ArrayList<MenuBtn> rightMenuList = new ArrayList<>();
        // 新增
        topMenuList.add(MenuBtn.topMenu_addNew());
        // 批量删除
        topMenuList.add(MenuBtn.topMenu_batchDelete());
        btnDatas.put("topMenu", topMenuList);
        // 新增
        rightMenuList.add(MenuBtn.rightMenu_addNew());
        // 日志
        rightMenuList.add(MenuBtn.rightMenu_btnLog());
        btnDatas.put("rightMenu", rightMenuList);
        return btnDatas;
    }

    @Override
    public Map<String, Object> getCompBaseForm(Map<String, Object> params) {
        OrganizationAssert.notNull(params.get("viewAttr"), "请标识操作类型");

        // 2编辑 1查看
        int viewAttr = Integer.parseInt((String) params.get("viewAttr"));
        long id = Long.parseLong((String) params.get("id"));
        String groupId = (String) params.get("viewCondition");

        HashMap<String, Object> buttonsMap = new HashMap<>();
        buttonsMap.put("hasEdit", true);
        buttonsMap.put("hasSave", true);

        List<SearchConditionGroup> addGroups = new ArrayList<>();
        if (StringUtil.isEmpty(groupId) || "0".equals(groupId)) {
            addGroups.add(new SearchConditionGroup("基本信息", true, getBaseForm(viewAttr, id)));
        } else {
            addGroups.add(new SearchConditionGroup(getExtendGroupMapper().getGroupNameById(groupId), true, getExtService(user).getExtForm(user, EXTEND_TYPE, JCL_ORG_COMPEXT, viewAttr, id, groupId)));
        }

        HashMap<String, Object> resultMap = new HashMap<>();
        resultMap.put("buttons", buttonsMap);
        resultMap.put("conditions", addGroups);
        resultMap.put("id", id);
        // 拓展页面分组
        resultMap.put("tabInfo", getExtService(user).getTabInfo(EXTEND_TYPE, JCL_ORG_COMPEXT));
        // 处理明细表
        resultMap.put("tables", getExtService(user).getExtendTables(user, EXTEND_TYPE, JCL_ORG_COMPEXT_DT1, id, viewAttr, false));
        Map<String, Object> apiDatas = new HashMap<>();

        apiDatas.put("result", resultMap);

        return apiDatas;
    }


    @Override
    public Map<String, Object> getCompSaveForm() {
        Map<String, Object> apiDatas = new HashMap<>();
        List<SearchConditionGroup> addGroups = new ArrayList<>();
        List<SearchConditionItem> conditionItems = new ArrayList<>();
        // 编号
        SearchConditionItem compNoItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 3, 50, "编号", "compNo");
        compNoItem.setRules("required|string");
        // 名称
        SearchConditionItem compNameItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 3, 50, "名称", "compName");
        compNameItem.setRules("required|string");
        // 简称
        SearchConditionItem compNameShortItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 3, 50, "简称", "compNameShort");
        compNameShortItem.setRules("required|string");
        // 上级公司
        SearchConditionItem compBrowserItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 2, false, "上级公司", "161", "parentCompany", "compBrowser");
        // 组织机构代码
        SearchConditionItem orgCodeItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 3, 50, "组织机构代码", "orgCode");
        orgCodeItem.setRules("required|string");
        // 行业
        SearchConditionItem industryItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 2, false, "行业", "63", "industry", "");
        // 负责人
        SearchConditionItem compPrincipalItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 3, false, "负责人", "1", "compPrincipal", "");
        compPrincipalItem.setRules("required|string");
        // 说明
        SearchConditionItem descriptionItem = OrganizationFormItemUtil.textareaItem(user, 2, 16, true, 2, 60, "说明", "description");

        conditionItems.add(compNoItem);
        conditionItems.add(compNameItem);
        conditionItems.add(compNameShortItem);
        conditionItems.add(compBrowserItem);
        conditionItems.add(orgCodeItem);
        conditionItems.add(industryItem);
        conditionItems.add(compPrincipalItem);
        conditionItems.add(descriptionItem);


        addGroups.add(new SearchConditionGroup("基本信息", true, conditionItems));
        apiDatas.put("condition", addGroups);

        return apiDatas;

    }

    /**
     * 是否为搜索查询
     *
     * @param compPO
     * @return
     */
    private boolean isFilter(CompPO compPO) {
        return !(StringUtil.isEmpty(compPO.getCompName())
                && StringUtil.isEmpty(compPO.getCompNo())
                && StringUtil.isEmpty(compPO.getCompNameShort())
                && StringUtil.isEmpty(compPO.getOrgCode())
                && null == compPO.getParentCompany()
                && null == compPO.getIndustry()
                && null == compPO.getCompPrincipal()
                && null == compPO.getForbiddenTag());
    }


    /**
     * 浏览按钮类型赋值转换
     *
     * @param id
     * @param name
     * @return
     */
    private List<Map<String, Object>> creatReplaceDatas(Object id, Object name) {
        List<Map<String, Object>> datas = new ArrayList<>();
        Map<String, Object> map = new HashMap<>();
        map.put("id", id);
        map.put("name", name);
        datas.add(map);
        return datas;

    }


    /**
     * 基本信息基础表单
     *
     * @param viewAttr
     * @param id
     * @return
     */
    private List<SearchConditionItem> getBaseForm(int viewAttr, long id) {
        List<SearchConditionItem> conditionItems = new ArrayList<>();
        // 编号
        SearchConditionItem compNoItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 1, 50, "编号", "compNo");
        // 名称
        SearchConditionItem compNameItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 3, 50, "名称", "compName");
        compNameItem.setRules("required|string");
        // 简称
        SearchConditionItem compNameShortItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 3, 50, "简称", "compNameShort");
        compNameShortItem.setRules("required|string");
        // TODO 自定义按钮
        // 上级公司
        SearchConditionItem compBrowserItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 2, false, "上级公司", "161", "parentCompany", "compBrowser");
        // 组织机构代码
        SearchConditionItem orgCodeItem = OrganizationFormItemUtil.inputItem(user, 2, 16, 2, 50, "组织机构代码", "orgCode");
        orgCodeItem.setRules("required|string");
        // 行业
        SearchConditionItem industryItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 2, false, "行业", "63", "industry", "");
        // 负责人
        SearchConditionItem compPrincipalItem = OrganizationFormItemUtil.browserItem(user, 2, 16, 2, false, "负责人", "1", "compPrincipal", "");
        compPrincipalItem.setRules("required|string");
        // 说明
        SearchConditionItem descriptionItem = OrganizationFormItemUtil.textareaItem(user, 2, 16, true, 2, 60, "说明", "description");


        conditionItems.add(compNoItem);
        conditionItems.add(compNameItem);
        conditionItems.add(compNameShortItem);
        conditionItems.add(compBrowserItem);
        conditionItems.add(orgCodeItem);
        conditionItems.add(industryItem);
        conditionItems.add(compPrincipalItem);
        conditionItems.add(descriptionItem);


        // 编辑、查看状态赋值,设置只读状态

        // 赋值
        CompPO compPO = getCompMapper().listById(id);
        OrganizationAssert.notNull(compPO, "数据不存在或数据已删除");
        compNoItem.setValue(compPO.getCompNo());
        compNameItem.setValue(compPO.getCompName());
        compNameShortItem.setValue(compPO.getCompNameShort());

        // compBrowserItem
        if (null != compPO.getParentCompany()) {
            BrowserBean compBrowserBean = compBrowserItem.getBrowserConditionParam();
            compBrowserBean.setReplaceDatas(creatReplaceDatas(compPO.getParentCompany(), getCompMapper().listById(compPO.getParentCompany()).getCompName()));
            compBrowserItem.setBrowserConditionParam(compBrowserBean);
        }
        orgCodeItem.setValue(compPO.getOrgCode());
        // industryItem
        BrowserBean industryBean = industryItem.getBrowserConditionParam();
        industryBean.setReplaceDatas(creatReplaceDatas(compPO.getIndustry(), new SectorInfoComInfo().getSectorInfoname(compPO.getIndustry() + "")));
        industryItem.setBrowserConditionParam(industryBean);
        try {
            // compPrincipalItem
            BrowserBean PrincipalBean = compPrincipalItem.getBrowserConditionParam();
            PrincipalBean.setReplaceDatas(creatReplaceDatas(compPO.getCompPrincipal(), new ResourceComInfo().getLastname(compPO.getCompPrincipal() + "")));
            compPrincipalItem.setBrowserConditionParam(PrincipalBean);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        descriptionItem.setValue(compPO.getDescription());

        // 查看，全部置位只读
        if (1 == viewAttr) {
            for (SearchConditionItem item : conditionItems) {
                item.setViewAttr(viewAttr);
            }
        }
        return conditionItems;
    }


}
